# Generated by Mustafa on 2023-01-30 15:30

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('event', '0002_auto_20200403_1401'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='meta_data',
            field=models.JSONField(blank=True, encoder=django.core.serializers.json.DjangoJSONEncoder, null=True),
        ),
    ]
