# Generated by Django 2.2.5 on 2020-04-03 14:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('event', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pushnotification',
            name='callback_url',
            field=models.URLField(max_length=2000),
        ),
    ]
